/*
 *   Copyright (C) 2008-2009 Anders Aagaard <aagaande@gmail.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"

FSSettings loadConfig()
{
    KConfigGroup cfg = KSharedConfig::openConfig("krunnerrc")->group("fsrunner");
    QStringList folderList = cfg.readEntry("paths", QStringList());
    QList<Folder> folders;
    for (int i = 0; i < folderList.size(); i++) {
        QStringList splitFolders = folderList.at(i).split(",");
        Folder f;
        f.path = QString();
        f.subdirs = 3;
        f.priority = 0;
        f.files = true;
        f.slow = false;
        if (splitFolders.size() == 0)
            continue;
        if (splitFolders.size() >= 1)
            f.path = splitFolders.at(0);
        if (splitFolders.size() >= 2)
            f.subdirs = splitFolders.at(1).toInt();
        if (splitFolders.size() >= 3)
            f.priority = splitFolders.at(2).toInt();
        if (splitFolders.size() >= 4)
            f.files = splitFolders.at(3).toInt() > 0;
        if (splitFolders.size() >= 5)
            f.slow = splitFolders.at(4).toInt() > 0;

        if (!f.path.isEmpty()) {
            QDir tmpPath(f.path);
            if (tmpPath.makeAbsolute())
                f.path = tmpPath.absolutePath();

            qDebug() << "Loaded path" << f.path << f.subdirs << f.priority << f.files << f.slow;
            folders.append(f);
        }
    }
    qDebug() << "Loaded" << folders.size() << "paths";
    FSSettings settings;
    settings.dbversion = cfg.readEntry("dbversion",0);
    settings.folders = folders;

    settings.treatSpaceAsStar = cfg.readEntry("treatSpaceAsStar",true);
    settings.searchTermStrict = cfg.readEntry("searchTermStrict",true);
    settings.searchFieldName = cfg.readEntry("searchFieldName",true);
    settings.enableActions = cfg.readEntry("enableActions",false);
    qDebug() << "Load info" << settings.treatSpaceAsStar << settings.searchTermStrict << settings.searchFieldName << settings.enableActions ;
    return settings;
}

static void savePaths(QList<Folder> folders)
{
    qDebug() << "savePaths";
    KConfigGroup cfg = KSharedConfig::openConfig("krunnerrc")->group("fsrunner");

    QStringList folderList;
    for (int i = 0; i < folders.size(); i++) {
        folderList.append(QString("%1, %2, %3, %4, %5").arg(folders.at(i).path).arg(folders.at(i).subdirs).arg(folders.at(i).priority).arg(folders.at(i).files).arg(folders.at(i).slow));
        qDebug() << "   " << folders.at(i).path << folders.at(i).subdirs << folders.at(i).priority << folders.at(i).files << folders.at(i).slow;
    }

    qDebug() << "Saving" << folderList.size() << "paths";
    cfg.writeEntry("paths", folderList);
}

void saveSettings(const FSSettings &settings)
{
    qDebug() << "Saving settings";
    KConfigGroup cfg = KSharedConfig::openConfig("krunnerrc")->group("fsrunner");
    savePaths(settings.folders);
    qDebug() << settings.treatSpaceAsStar << settings.searchTermStrict << settings.enableActions ;
    cfg.writeEntry("treatSpaceAsStar", settings.treatSpaceAsStar);
    cfg.writeEntry("searchTermStrict", settings.searchTermStrict);
    cfg.writeEntry("searchFieldName", settings.searchFieldName);
    cfg.writeEntry("enableActions", settings.enableActions);

    cfg.sync();
}

void saveVersion(QString key, int version)
{
    assert(key.size() > 0);
    assert(version > 0);
    KConfigGroup cfg = KSharedConfig::openConfig("krunnerrc")->group("fsrunner");
    cfg.writeEntry(key, version);
    cfg.sync();
}
