/*
 *   Copyright (C) 2008-2009 Anders Aagaard <aagaande@gmail.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef Database_H
#define Database_H

#include <QtSql>
#include <QList>
#include <QMessageBox>

#include <KDebug>
#include <KIcon>
#include <kmimetype.h>

#include <Plasma/AbstractRunner>

#include <cassert>

#include "config.h"

#define QUERY_EXEC(q) if (!q) { qFatal("Error in query: %s", qPrintable(query.lastError().text())); exit(1); }
#define QUERY_MATCH(q) if (!q) { qFatal("Error in query : %s, query '%s' -> '%s'\nSQL : '%s'\nPlease report this to the issue tracker.",qPrintable(query.lastError().text()), qPrintable(matchString), qPrintable(m), qPrintable(query.executedQuery())); exit(1); }
#define CONNECTIONNAME "fsRunnerDb"

static bool database_initialized = false;
static QSqlDatabase connection_;

class Database
{
private:
    bool transaction_;
    QMutex mutex_;

    // Loaded from settings:
    bool treatSpaceAsStar;
    bool searchTermStrict;
    bool searchFieldName;

    // These functions are not threadsafe, and are therefor private:
    void beginTransaction();
    void endTransaction();

    void createTables();
public:
    Database(FSSettings *settings);
    ~Database();
    bool isValid();

    void cleanDatabase(FSSettings *settings);
    void forceCommit();
    const QHash<QString, int> getEntries(const int parent);
#ifndef LOWMEM_SCAN
    const QVarLengthArray<QHash<QString, int> > getEntries();
#endif
    int addEntry(QString name, QString absolutePath, bool directory, int priority, int parent);
    int getEntry(QString absolutePath);
    void remove(QList<int> ids);
    void remove(QString absolutePath);
    const QList<Plasma::QueryMatch> match(Plasma::AbstractRunner *runner, QString match);
};
#endif
