/*
 *   Copyright (C) 2008-2009 Anders Aagaard <aagaande@gmail.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef FS_H
#define FS_H

#include <QMenu>

#include <KGenericFactory>
#include <KRun>
#include <KIO/NetAccess>

//libkonq
#include <konq_popupmenuinformation.h>
#include <konq_popupmenu.h>
#include <knewmenu.h>
#include <konq_menuactions.h>

#include <plasma/abstractrunner.h>
#include "database.h"
#include "dirwatcher.h"

/**
 * This monitors the filesystem and allows searches for directory names/files.
 */
class FSRunner : public Plasma::AbstractRunner
{
Q_OBJECT
private:
    Database *database_;
    DirWatcher *dirWatcher_;
    KonqMenuActions *actions_;
    bool enableActions_;
    int actionsAdded_;

public:
    FSRunner(QObject* parent, const QVariantList &args);
    ~FSRunner();

    void match(Plasma::RunnerContext &context);
    void run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &action);
    void reloadConfiguration();
    QList<QAction*> actionsForMatch(const Plasma::QueryMatch &match);
    QList<QAction*> parseMenu(QMenu *menu, const QString &prefix = QString());
};

K_EXPORT_PLASMA_RUNNER(fsrunner, FSRunner)

#endif
