/*
 *   Copyright (C) 2008-2009 Anders Aagaard <aagaande@gmail.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "ioprio.h"

#include <stdio.h>
#include <errno.h>
#include <sys/syscall.h>
#include <unistd.h>
#include <QDebug>

#ifdef SYS_ioprio_set

#ifndef __NR_ioprio_set
#if defined(__i386__)
#define __NR_ioprio_set         289
#define __NR_ioprio_get         290
#elif defined(__powerpc__) || defined(__powerpc64__)
#define __NR_ioprio_set         273
#define __NR_ioprio_get         274
#elif defined(__x86_64__)
#define __NR_ioprio_set         251
#define __NR_ioprio_get         252
#elif defined(__ia64__)
#define __NR_ioprio_set         1274
#define __NR_ioprio_get         1275
#elif defined(__alpha__)
#define __NR_ioprio_set         442
#define __NR_ioprio_get         443
#elif defined(__s390x__) || defined(__s390__)
#define __NR_ioprio_set         282
#define __NR_ioprio_get         283
#elif defined(__SH4__)
#define __NR_ioprio_set         288
#define __NR_ioprio_get         289
#elif defined(__SH5__)
#define __NR_ioprio_set         316
#define __NR_ioprio_get         317
#elif defined(__sparc__) || defined(__sparc64__)
#define __NR_ioprio_set         196
#define __NR_ioprio_get         218
#elif defined(__arm__)
#define __NR_ioprio_set         314
#define __NR_ioprio_get         315
#else
#error "Unsupported architecture!"
#endif
#endif

enum {
        IOPRIO_CLASS_NONE,
        IOPRIO_CLASS_RT,
        IOPRIO_CLASS_BE,
        IOPRIO_CLASS_IDLE,
};

enum {
        IOPRIO_WHO_PROCESS = 1,
        IOPRIO_WHO_PGRP,
        IOPRIO_WHO_USER,
};

#define IOPRIO_CLASS_SHIFT  13

static inline int ioprio_set(int which, int who, int ioprio)
{
	return syscall(__NR_ioprio_set, which, who, ioprio);
}

void ioprio()
{
    qDebug() << "Idle ioprio set";
    if (ioprio_set (IOPRIO_WHO_PROCESS, 0, IOPRIO_CLASS_IDLE<<IOPRIO_CLASS_SHIFT) == -1) {
        perror("ioprio_set");
    }
}
#else
void ioprio()
{
    qDebug() << "Ioprio not set";
}
#endif
